import logging

from pytest import fixture, skip
from databricks.sdk import WorkspaceClient

logger = logging.getLogger(__name__)


@fixture
def spark(ws: WorkspaceClient):
    """
    Get Databricks Connect Spark session. Requires `databricks-connect` package to be installed.

    Usage:
    ```python
    def test_databricks_connect(spark):
        rows = spark.sql("SELECT 1").collect()
        assert rows[0][0] == 1
    ```
    """
    if not ws.config.cluster_id:
        skip("No cluster_id found in the environment")
    ws.clusters.ensure_cluster_is_running(ws.config.cluster_id)
    try:
        # pylint: disable-next=import-outside-toplevel
        from databricks.connect import (  # type: ignore[import-untyped]
            DatabricksSession,
        )

        return DatabricksSession.builder.sdkConfig(ws.config).getOrCreate()
    except ImportError:
        skip("Please run `pip install databricks-connect`")
        return None



import json
import warnings
from collections.abc import Callable, Generator
from pathlib import Path
from unittest.mock import Mock

from pytest import fixture
from databricks.sdk.service._internal import Wait
from databricks.sdk.service.compute import (
    CreatePolicyResponse,
    ClusterDetails,
    ClusterSpec,
    CreateInstancePoolResponse,
    Library,
)
from databricks.sdk.service.jobs import Job, JobSettings, NotebookTask, SparkPythonTask, Task
from databricks.sdk.service.pipelines import CreatePipelineResponse, PipelineLibrary, NotebookLibrary, PipelineCluster
from databricks.sdk.service.sql import (
    CreateWarehouseRequestWarehouseType,
    EndpointTags,
    EndpointTagPair,
    GetWarehouseResponse,
)

from databricks.labs.pytester.fixtures.baseline import factory


@fixture
def make_cluster_policy(
    ws,
    make_random,
    log_workspace_link,
    watchdog_purge_suffix,
) -> Generator[Callable[..., CreatePolicyResponse], None, None]:
    """
    Create a Databricks cluster policy and clean it up after the test. Returns a function to create cluster policies,
    which returns `databricks.sdk.service.compute.CreatePolicyResponse` instance.

    Keyword Arguments:
    * `name` (str, optional): The name of the cluster policy. If not provided, a random name will be generated.

    Usage:
    ```python
    def test_cluster_policy(make_cluster_policy):
        logger.info(f"created {make_cluster_policy()}")
    ```
    """

    def create(*, name: str | None = None, **kwargs) -> CreatePolicyResponse:
        if name is None:
            name = f"dummy-{make_random(4)}-{watchdog_purge_suffix}"
        if "definition" not in kwargs:
            kwargs["definition"] = json.dumps(
                {
                    "spark_conf.spark.databricks.delta.preview.enabled": {"type": "fixed", "value": "true"},
                }
            )
        cluster_policy = ws.cluster_policies.create(name=name, **kwargs)
        log_workspace_link(name, f'setting/clusters/cluster-policies/view/{cluster_policy.policy_id}', anchor=False)
        return cluster_policy

    yield from factory("cluster policy", create, lambda item: ws.cluster_policies.delete(item.policy_id))


@fixture
def make_cluster(
    ws, make_random, log_workspace_link, watchdog_remove_after
) -> Generator[Callable[..., Wait[ClusterDetails]], None, None]:
    """
    Create a Databricks cluster, waits for it to start, and clean it up after the test.
    Returns a function to create clusters. You can get `cluster_id` attribute from the returned object.

    Keyword Arguments:
    * `single_node` (bool, optional): Whether to create a single-node cluster. Defaults to False.
    * `cluster_name` (str, optional): The name of the cluster. If not provided, a random name will be generated.
    * `spark_version` (str, optional): The Spark version of the cluster. If not provided, the latest version will be used.
    * `autotermination_minutes` (int, optional): The number of minutes before the cluster is automatically terminated. Defaults to 10.

    Usage:
    ```python
    def test_cluster(make_cluster):
        logger.info(f"created {make_cluster(single_node=True)}")
    ```
    """

    def create(
        *,
        single_node: bool = False,
        cluster_name: str | None = None,
        spark_version: str | None = None,
        autotermination_minutes=10,
        **kwargs,
    ) -> Wait[ClusterDetails]:
        if cluster_name is None:
            cluster_name = f"dummy-{make_random(4)}"
        if spark_version is None:
            spark_version = ws.clusters.select_spark_version(latest=True)
        if single_node:
            kwargs["num_workers"] = 0
            if "spark_conf" in kwargs:
                kwargs["spark_conf"] = kwargs["spark_conf"] | {
                    "spark.databricks.cluster.profile": "singleNode",
                    "spark.master": "local[*]",
                }
            else:
                kwargs["spark_conf"] = {"spark.databricks.cluster.profile": "singleNode", "spark.master": "local[*]"}
            kwargs["custom_tags"] = {"ResourceClass": "SingleNode"}
        if "instance_pool_id" not in kwargs:
            kwargs["node_type_id"] = ws.clusters.select_node_type(local_disk=True, min_memory_gb=16)
        if "custom_tags" not in kwargs:
            kwargs["custom_tags"] = {"RemoveAfter": watchdog_remove_after}
        else:
            kwargs["custom_tags"]["RemoveAfter"] = watchdog_remove_after
        wait = ws.clusters.create(
            cluster_name=cluster_name,
            spark_version=spark_version,
            autotermination_minutes=autotermination_minutes,
            **kwargs,
        )
        log_workspace_link(cluster_name, f'compute/clusters/{wait.cluster_id}', anchor=False)
        return wait

    yield from factory("cluster", create, lambda item: ws.clusters.permanent_delete(item.cluster_id))


@fixture
def make_instance_pool(
    ws,
    make_random,
    log_workspace_link,
    watchdog_remove_after,
) -> Generator[Callable[..., CreateInstancePoolResponse], None, None]:
    """
    Create a Databricks instance pool and clean it up after the test. Returns a function to create instance pools.
    Use `instance_pool_id` attribute from the returned object to get an ID of the pool.

    Keyword Arguments:
    * `instance_pool_name` (str, optional): The name of the instance pool. If not provided, a random name will be generated.
    * `node_type_id` (str, optional): The node type ID of the instance pool. If not provided, a node type with local disk and 16GB memory will be used.
    * other arguments are passed to `WorkspaceClient.instance_pools.create` method.

    Usage:
    ```python
    def test_instance_pool(make_instance_pool):
        logger.info(f"created {make_instance_pool()}")
    ```
    """

    def create(*, instance_pool_name=None, node_type_id=None, **kwargs) -> CreateInstancePoolResponse:
        if instance_pool_name is None:
            instance_pool_name = f"dummy-{make_random(4)}"
        if node_type_id is None:
            node_type_id = ws.clusters.select_node_type(local_disk=True, min_memory_gb=16)
        pool = ws.instance_pools.create(
            instance_pool_name,
            node_type_id,
            custom_tags={"RemoveAfter": watchdog_remove_after},
            **kwargs,
        )
        log_workspace_link(instance_pool_name, f'compute/instance-pools/{pool.instance_pool_id}', anchor=False)
        return pool

    yield from factory("instance pool", create, lambda pool: ws.instance_pools.delete(pool.instance_pool_id))


@fixture
def make_job(
    ws,
    make_random,
    make_notebook,
    make_workspace_file,
    log_workspace_link,
    watchdog_remove_after,
) -> Generator[Callable[..., Job], None, None]:
    """
    Create a Databricks job and clean it up after the test. Returns a function to create jobs, that returns
    a `databricks.sdk.service.jobs.Job` instance.

    Keyword Arguments:
    * `name` (str, optional): The name of the job. If not provided, a random name will be generated.
    * `path` (str, optional): The path to the notebook or file used in the job. If not provided, a random notebook or file will be created
    * [DEPRECATED: Use `path` instead] `notebook_path` (str, optional): The path to the notebook. If not provided, a random notebook will be created.
    * `content` (str | bytes, optional): The content of the notebook or file used in the job. If not provided, default content of `make_notebook` will be used.
    * `task_type` (type[NotebookTask] | type[SparkPythonTask], optional): The type of task. If not provides, `type[NotebookTask]` will be used.
    * `instance_pool_id` (str, optional): The instance pool id to add to the job cluster. If not provided, no instance pool will be used.
    * `spark_conf` (dict, optional): The Spark configuration of the job. If not provided, Spark configuration is not explicitly set.
    * `libraries` (list, optional): The list of libraries to install on the job.
    * `tags` (list[str], optional): A list of job tags. If not provided, no additional tags will be set on the job.
    * `tasks` (list[Task], optional): A list of job tags. If not provided, a single task with a notebook task will be
       created, along with a disposable notebook. Latest Spark version and a single worker clusters will be used to run
       this ephemeral job.

    Usage:
    ```python
    def test_job(make_job):
        logger.info(f"created {make_job()}")
    ```
    """

    def create(  # pylint: disable=too-many-arguments
        *,
        name: str | None = None,
        path: str | Path | None = None,
        notebook_path: str | Path | None = None,  # DEPRECATED
        content: str | bytes | None = None,
        task_type: type[NotebookTask] | type[SparkPythonTask] = NotebookTask,
        spark_conf: dict[str, str] | None = None,
        instance_pool_id: str | None = None,
        libraries: list[Library] | None = None,
        tags: dict[str, str] | None = None,
        tasks: list[Task] | None = None,
    ) -> Job:
        if notebook_path is not None:
            warnings.warn(
                "The `notebook_path` parameter is replaced with the more general `path` parameter "
                "when introducing workspace paths to python scripts.",
                DeprecationWarning,
            )
            path = notebook_path
        if path and content:
            raise ValueError("The `path` and `content` parameters are exclusive.")
        if tasks and (path or content or spark_conf or libraries):
            raise ValueError(
                "The `tasks` parameter is exclusive with the `path`, `content` `spark_conf` and `libraries` parameters."
            )
        name = name or f"dummy-j{make_random(4)}"
        tags = tags or {}
        tags["RemoveAfter"] = tags.get("RemoveAfter", watchdog_remove_after)
        if not tasks:
            node_type_id = None
            if instance_pool_id is None:
                node_type_id = ws.clusters.select_node_type(local_disk=True, min_memory_gb=16)
            task = Task(
                task_key=make_random(4),
                description=make_random(4),
                new_cluster=ClusterSpec(
                    num_workers=1,
                    node_type_id=node_type_id,
                    spark_version=ws.clusters.select_spark_version(latest=True),
                    instance_pool_id=instance_pool_id,
                    spark_conf=spark_conf,
                ),
                libraries=libraries,
                timeout_seconds=0,
            )
            if task_type == SparkPythonTask:
                path = path or make_workspace_file(content=content)
                task.spark_python_task = SparkPythonTask(python_file=str(path))
            else:
                path = path or make_notebook(content=content)
                task.notebook_task = NotebookTask(notebook_path=str(path))
            tasks = [task]
        response = ws.jobs.create(name=name, tasks=tasks, tags=tags)
        log_workspace_link(name, f"job/{response.job_id}", anchor=False)
        job = ws.jobs.get(response.job_id)
        if isinstance(response, Mock):  # For testing
            job = Job(settings=JobSettings(name=name, tasks=tasks, tags=tags))
        return job

    yield from factory("job", create, lambda item: ws.jobs.delete(item.job_id))


@fixture
def make_pipeline(
    ws,
    make_random,
    make_notebook,
    watchdog_remove_after,
    watchdog_purge_suffix,
) -> Generator[Callable[..., CreatePipelineResponse], None, None]:
    """
    Create Delta Live Table Pipeline and clean it up after the test. Returns a function to create pipelines.
    Results in a `databricks.sdk.service.pipelines.CreatePipelineResponse` instance.

    Keyword Arguments:
    * `name` (str, optional): The name of the pipeline. If not provided, a random name will be generated.
    * `libraries` (list, optional): The list of libraries to install on the pipeline. If not provided, a random disposable notebook will be created.
    * `clusters` (list, optional): The list of clusters to use for the pipeline. If not provided, a single node cluster will be created with 16GB memory and local disk.

    Usage:
    ```python
    def test_pipeline(make_pipeline, make_pipeline_permissions, make_group):
        group = make_group()
        pipeline = make_pipeline()
        make_pipeline_permissions(
            object_id=pipeline.pipeline_id,
            permission_level=PermissionLevel.CAN_MANAGE,
            group_name=group.display_name,
        )
    ```
    """

    def create(**kwargs) -> CreatePipelineResponse:
        if "name" not in kwargs:
            kwargs["name"] = f"sdk-{make_random(4)}-{watchdog_purge_suffix}"
        if "libraries" not in kwargs:
            notebook_library = NotebookLibrary(path=make_notebook().as_posix())
            kwargs["libraries"] = [PipelineLibrary(notebook=notebook_library)]
        if "clusters" not in kwargs:
            kwargs["clusters"] = [
                PipelineCluster(
                    node_type_id=ws.clusters.select_node_type(local_disk=True, min_memory_gb=16),
                    label="default",
                    num_workers=1,
                    custom_tags={"cluster_type": "default", "RemoveAfter": watchdog_remove_after},
                )
            ]
        return ws.pipelines.create(continuous=False, **kwargs)

    yield from factory("delta live table", create, lambda item: ws.pipelines.delete(item.pipeline_id))


@fixture
def make_warehouse(
    ws, make_random, watchdog_remove_after
) -> Generator[Callable[..., Wait[GetWarehouseResponse]], None, None]:
    """
    Create a Databricks warehouse and clean it up after the test. Returns a function to create warehouses.

    Keyword Arguments:
    * `warehouse_name` (str, optional): The name of the warehouse. If not provided, a random name will be generated.
    * `warehouse_type` (CreateWarehouseRequestWarehouseType, optional): The type of the warehouse. Defaults to `PRO`.
    * `cluster_size` (str, optional): The size of the cluster. Defaults to `2X-Small`.

    Usage:
    ```python
    def test_warehouse_has_remove_after_tag(ws, make_warehouse):
        new_warehouse = make_warehouse()
        created_warehouse = ws.warehouses.get(new_warehouse.response.id)
        warehouse_tags = created_warehouse.tags.as_dict()
        assert warehouse_tags["custom_tags"][0]["key"] == "RemoveAfter"
    ```
    """

    def create(
        *,
        warehouse_name: str | None = None,
        warehouse_type: CreateWarehouseRequestWarehouseType | None = None,
        cluster_size: str | None = None,
        max_num_clusters: int = 1,
        enable_serverless_compute: bool = False,
        **kwargs,
    ) -> Wait[GetWarehouseResponse]:
        if warehouse_name is None:
            warehouse_name = f"dummy-{make_random(4)}"
        if warehouse_type is None:
            warehouse_type = CreateWarehouseRequestWarehouseType.PRO
        if cluster_size is None:
            cluster_size = "2X-Small"

        remove_after_tags = EndpointTags(custom_tags=[EndpointTagPair(key="RemoveAfter", value=watchdog_remove_after)])
        return ws.warehouses.create(
            name=warehouse_name,
            cluster_size=cluster_size,
            warehouse_type=warehouse_type,
            max_num_clusters=max_num_clusters,
            enable_serverless_compute=enable_serverless_compute,
            tags=remove_after_tags,
            **kwargs,
        )

    yield from factory("warehouse", create, lambda item: ws.warehouses.delete(item.id))



import logging
import warnings
from collections.abc import Callable, Generator
from datetime import timedelta

from pytest import fixture
from databricks.sdk import AccountGroupsAPI, GroupsAPI, WorkspaceClient
from databricks.sdk.config import Config
from databricks.sdk.errors import ResourceConflict, NotFound
from databricks.sdk.retries import retried
from databricks.sdk.service import iam
from databricks.sdk.service.iam import User, Group

from databricks.labs.pytester.fixtures.baseline import factory

logger = logging.getLogger(__name__)


@fixture
def make_user(ws, make_random, log_workspace_link, watchdog_purge_suffix):
    """
    This fixture returns a function that creates a Databricks workspace user
    and removes it after the test is complete. In case of random naming conflicts,
    the fixture will retry the creation process for 30 seconds. Returns an instance
    of `databricks.sdk.service.iam.User`. Usage:

    ```python
    def test_new_user(make_user, ws):
        new_user = make_user()
        home_dir = ws.workspace.get_status(f"/Users/{new_user.user_name}")
        assert home_dir.object_type == ObjectType.DIRECTORY
    ```
    """

    @retried(on=[ResourceConflict], timeout=timedelta(seconds=30))
    def create(**kwargs) -> User:
        user_name = f"dummy-{make_random(4)}-{watchdog_purge_suffix}@example.com".lower()
        user = ws.users.create(user_name=user_name, **kwargs)
        log_workspace_link(user.user_name, f'settings/workspace/identity-and-access/users/{user.id}')
        return user

    yield from factory("workspace user", create, lambda item: ws.users.delete(item.id))


@fixture
def make_group(ws: WorkspaceClient, make_random, watchdog_purge_suffix):
    """
    This fixture provides a function to manage Databricks workspace groups. Groups can be created with specified
    members and roles, and they will be deleted after the test is complete. Deals with eventual consistency issues by
    retrying the creation process for 30 seconds and then waiting for up to 3 minutes for the group to be provisioned.
    Returns an instance of `databricks.sdk.service.iam.Group`.

    Keyword arguments:
    * `members` (list of strings): A list of user IDs to add to the group.
    * `roles` (list of strings): A list of roles to assign to the group.
    * `display_name` (str): The display name of the group.
    * `entitlements` (list of strings): A list of entitlements to assign to the group.

    The following example creates a group with a single member and independently verifies that the group was created:

    ```python
    def test_new_group(make_group, make_user, ws):
        user = make_user()
        group = make_group(members=[user.id])
        loaded = ws.groups.get(group.id)
        assert group.display_name == loaded.display_name
        assert group.members == loaded.members
    ```
    """
    yield from _make_group("workspace group", ws.config, ws.groups, make_random, watchdog_purge_suffix)


@fixture
def make_acc_group(acc, make_random, watchdog_purge_suffix):
    """
    This fixture provides a function to manage Databricks account groups. Groups can be created with
    specified members and roles, and they will be deleted after the test is complete.

    Has the same arguments and behavior as [`make_group` fixture](#make_group-fixture) but uses the account
    client instead of the workspace client.

    Example usage:
    ```python
    def test_new_account_group(make_acc_group, acc):
        group = make_acc_group()
        loaded = acc.groups.get(group.id)
        assert group.display_name == loaded.display_name
    ```
    """
    yield from _make_group("account group", acc.config, acc.groups, make_random, watchdog_purge_suffix)


def _scim_values(ids: list[str]) -> list[iam.ComplexValue]:
    return [iam.ComplexValue(value=x) for x in ids]


def _wait_group_provisioned(interface: AccountGroupsAPI | GroupsAPI, group: Group) -> None:
    """Wait for a group to be visible via the supplied group interface.

    Due to consistency issues in the group-management APIs, new groups are not always visible in a consistent manner
    after being created or modified. This method can be used to mitigate against this by checking that a group:

     - Is visible via the `.get()` interface;
     - Is visible via the `.list()` interface that enumerates groups.

    Visibility is assumed when 2 calls in a row return the expected results.

    Args:
          interface: the group-management interface to use for checking whether the groups are visible.
          group: the group whose visibility should be verified.
    Raises:
          NotFound: this is thrown if it takes longer than 90 seconds for the group to become visible via the
          management interface.
    """
    # Use double-checking to try and compensate for the lack of monotonic consistency with the group-management
    # interfaces: two subsequent calls need to succeed for us to proceed. (This is probabilistic, and not a guarantee.)
    # The REST API internals cache things for up to 60s, and we see times close to this during tests. The retry timeout
    # reflects this: if it's taking much longer then something else is wrong.
    group_id = group.id
    assert group_id is not None

    @retried(on=[NotFound], timeout=timedelta(seconds=90))
    def _double_get_group() -> None:
        interface.get(group_id)
        interface.get(group_id)

    def _check_group_in_listing() -> None:
        found_groups = interface.list(attributes="id", filter=f'id eq "{group_id}"')
        found_ids = {found_group.id for found_group in found_groups}
        if group_id not in found_ids:
            msg = f"Group id not (yet) found in group listing: {group_id}"
            raise NotFound(msg)

    @retried(on=[NotFound], timeout=timedelta(seconds=90))
    def _double_check_group_in_listing() -> None:
        _check_group_in_listing()
        _check_group_in_listing()

    _double_get_group()
    _double_check_group_in_listing()


def _make_group(
    name: str, cfg: Config, interface, make_random, watchdog_purge_suffix
) -> Generator[Callable[..., Group], None, None]:
    _not_specified = object()

    @retried(on=[ResourceConflict], timeout=timedelta(seconds=30))
    def create(
        *,
        members: list[str] | None = None,
        roles: list[str] | None = None,
        entitlements: list[str] | None = None,
        display_name: str | None = None,
        wait_for_provisioning: bool | object = _not_specified,
        **kwargs,
    ):
        kwargs["display_name"] = (
            f"sdk-{make_random(4)}-{watchdog_purge_suffix}" if display_name is None else display_name
        )
        if members is not None:
            kwargs["members"] = _scim_values(members)
        if roles is not None:
            kwargs["roles"] = _scim_values(roles)
        if entitlements is not None:
            kwargs["entitlements"] = _scim_values(entitlements)
        if wait_for_provisioning is not _not_specified:
            warnings.warn(
                "Specifying wait_for_provisioning when making a group is deprecated; we always wait.",
                DeprecationWarning,
                # Call stack is: create()[iam.py], wrapper()[retries.py], inner()[baseline.py], client_code
                stacklevel=4,
            )
        # TODO: REQUEST_LIMIT_EXCEEDED: GetUserPermissionsRequest RPC token bucket limit has been exceeded.
        group = interface.create(**kwargs)
        if cfg.is_account_client:
            logger.info(f"Account group {group.display_name}: {cfg.host}/users/groups/{group.id}/members")
        else:
            logger.info(f"Workspace group {group.display_name}: {cfg.host}#setting/accounts/groups/{group.id}")

        _wait_group_provisioned(interface, group)

        return group

    yield from factory(name, create, lambda item: interface.delete(item.id))



import json
import os
from pathlib import Path
import sys
from collections.abc import MutableMapping, Callable

import pytest
from pytest import fixture
from databricks.labs.blueprint.entrypoint import find_dir_with_leaf


@fixture
def is_in_debug() -> bool:
    """
    Returns true if the test is running from a debugger in IDE, otherwise false.

    The following IDE are supported: IntelliJ IDEA (including Community Edition),
    PyCharm (including Community Edition), and Visual Studio Code.
    """
    return os.path.basename(sys.argv[0]) in {"_jb_pytest_runner.py", "testlauncher.py"}


@fixture
def debug_env_name():
    """
    Specify the name of the debug environment. By default, it is set to `.env`,
    which will try to find a [file named `.env`](https://www.dotenv.org/docs/security/env)
    in any of the parent directories of the current working directory and load
    the environment variables from it via the [`debug_env` fixture](#debug_env-fixture).

    Alternatively, if you are concerned of the
    [risk of `.env` files getting checked into version control](https://thehackernews.com/2024/08/attackers-exploit-public-env-files-to.html),
    we recommend using the `~/.databricks/debug-env.json` file to store different sets of environment variables.
    The file cannot be checked into version control by design, because it is stored in the user's home directory.

    This file is used for local debugging and integration tests in IDEs like PyCharm, VSCode, and IntelliJ IDEA
    while developing Databricks Platform Automation Stack, which includes Databricks SDKs for Python, Go, and Java,
    as well as Databricks Terraform Provider and Databricks CLI. This file enables multi-environment and multi-cloud
    testing with a single set of integration tests.

    The file is typically structured as follows:

    ```shell
    $ cat ~/.databricks/debug-env.json
    {
       "ws": {
         "CLOUD_ENV": "azure",
         "DATABRICKS_HOST": "....azuredatabricks.net",
         "DATABRICKS_CLUSTER_ID": "0708-200540-...",
         "DATABRICKS_WAREHOUSE_ID": "33aef...",
            ...
       },
       "acc": {
         "CLOUD_ENV": "aws",
         "DATABRICKS_HOST": "accounts.cloud.databricks.net",
         "DATABRICKS_CLIENT_ID": "....",
         "DATABRICKS_CLIENT_SECRET": "....",
         ...
       }
    }
    ```

    And you can load it in your `conftest.py` file as follows:

    ```python
    @pytest.fixture
    def debug_env_name():
        return "ws"
    ```

    This will load the `ws` environment from the `~/.databricks/debug-env.json` file.

    If any of the environment variables are not found, [`env_or_skip` fixture](#env_or_skip-fixture)
    will gracefully skip the execution of tests.
    """
    return ".env"


@fixture
def debug_env(monkeypatch, debug_env_name, is_in_debug) -> MutableMapping[str, str]:
    """
    Loads environment variables specified in [`debug_env_name` fixture](#debug_env_name-fixture) from a file
    for local debugging in IDEs, otherwise allowing the tests to run with the default environment variables
    specified in the CI/CD pipeline.
    """
    if not is_in_debug:
        return os.environ
    if debug_env_name == ".env":
        dot_env = _parse_dotenv()
        if not dot_env:
            return os.environ
        for env_key, value in dot_env.items():
            monkeypatch.setenv(env_key, value)
        return os.environ
    conf_file = Path.home() / ".databricks/debug-env.json"
    if not conf_file.exists():
        return os.environ
    with conf_file.open("r") as f:
        conf = json.load(f)
        if debug_env_name not in conf:
            sys.stderr.write(f"""{debug_env_name} not found in ~/.databricks/debug-env.json""")
            msg = f"{debug_env_name} not found in ~/.databricks/debug-env.json"
            raise KeyError(msg)
        for env_key, value in conf[debug_env_name].items():
            monkeypatch.setenv(env_key, value)
    return os.environ


@fixture
def env_or_skip(debug_env, is_in_debug) -> Callable[[str], str]:
    """
    Fixture to get environment variables or skip tests.

    It is extremely useful to skip tests if the required environment variables are not set.

    In the following example, `test_something` would only run if the environment variable
    `SOME_EXTERNAL_SERVICE_TOKEN` is set:

    ```python
    def test_something(env_or_skip):
        token = env_or_skip("SOME_EXTERNAL_SERVICE_TOKEN")
        assert token is not None
    ```
    """
    skip = pytest.skip
    if is_in_debug:
        skip = pytest.fail  # type: ignore[assignment]

    def inner(var: str) -> str:
        if var not in debug_env:
            skip(f"Environment variable {var} is missing")
        return debug_env[var]

    return inner


def _parse_dotenv():
    """See https://www.dotenv.org/docs/security/env"""
    dot_env = find_dir_with_leaf(Path.cwd(), '.env')
    if dot_env is None:
        return {}
    env_vars = {}
    with (dot_env / '.env').open(encoding='utf8') as file:
        for line in file:
            line = line.strip()
            if not line or line.startswith('#'):
                continue
            if '=' not in line:
                continue
            key, value = line.split('=', 1)
            # Remove any surrounding quotes (single or double)
            value = value.strip().strip('"').strip("'")
            env_vars[key.strip()] = value
    return env_vars
